#############################################################################
# Samplesim is composed of four functions which allow investigating the     #
# effect of sample size on the outcome of mixing model estimations          #
# performed with siar. Allsources modified the sample sizes of all sources  #
# simultaneously, Onesource modifies the sample size on only one user       #
# defined source, and Consumer modifies the sample size for the consumer.   #
# In each case the simlations start with the mean and standard deviation    #
# for each source/consumer and isotope. The present version uses two        #
# isotopes. The last function, Plotsim,  plots the results of the           #
# simulations. Dorothee Ehrich & Nicolas Lecomte, march 2009.               #
#############################################################################

# function which calculates mean and sd for a group of isotope values
# and formats it so that it can be attached to the sourcesdemo file of SiaR.
# the start file are two columns with d15N and d13C values

meansd <- function(dat, Sources) {
  Meand15N <- mean(dat[ ,1])
  Meand13C <- mean(dat[ ,2])
  SDd15N <- sd(dat[ ,1])
  SDd13C <- sd(dat[ ,1])
  return(data.frame(Sources, Meand15N, SDd15N, Meand13C, SDd13C))
  }

# This function makes a smaller sample of a list of isotope measurements,
# calculates mean and sd, and formats it as an SiaR source file

meansd.rs <- function(dat, Sources, n) {
  iso1 <- sample(dat[ ,1], n)
  iso2 <- sample(dat[ ,2], n)
  Meand15N <- mean(iso1)
  Meand13C <- mean(iso2)
  SDd15N <- sd(iso1)
  SDd13C <- sd(iso2)
  return(data.frame(Sources, Meand15N, SDd15N, Meand13C, SDd13C))
  }

# This function makes a sample of isotope measurements based on the mean and
# Standard deviation and the assumption that measurements have a normal
# distribution. It calculates mean and sd, and formats it as an SiaR source file.
  
meansd.nd <- function(meand1, sdd1, meand2, sdd2, Sources="source1", n) {
  iso1 <- rnorm(n, meand1, sdd1)
  iso2 <- rnorm(n, meand2, sdd2)
  Meand15N <- mean(iso1)
  Meand13C <- mean(iso2)
  SDd15N <- sd(iso1)
  SDd13C <- sd(iso2)
  return(data.frame(Sources, Meand15N, SDd15N, Meand13C, SDd13C))
  }
  
# This function estimates mixtures modifying the sample sizes for
# all sources in the source file according to a normnal distribution and
# the means and standard deviations in the sources file.

credint <- function(dat, interval) {
   remov <- round((length(dat)-(interval/100)*length(dat)) / 2)
   return(c(sort(dat)[remov + 1], sort(dat)[length(dat) - remov - 1]))
   }

credint1 <- function(dat.df, interva) {
  apply(dat.df, 2, credint, interval=interva)
  }
    
allsources <- function(target, sours, nsamples, correct, nreplic, interv) {
  nbsources <- dim(sours)[1]

  res <- vector("list", nreplic)
  resamples <- vector("list", nreplic)
  for (n in 1:nreplic) {
  print(paste("calculates replicate data set", n), quote=FALSE)
  sours1 <- sours
  for (i in 1:nbsources) {
    sours1[i, ] <- meansd.nd(sours[i, 2], sours[i, 3], sours[i, 4], sours[i, 5], sours[i, 1], nsamples)
    }
  resamples[[n]] <- sours1
  res[[n]] <- siarmcmcdirichletv4(target, sours1, correct)[[15]]
  rm(sours1)  
  }

  credintervals <- lapply(res, credint1, interva=interv) #list with a matrix with 2 lines for each source
  precision <- matrix(NA, nreplic, nbsources)
  colnames(precision) <- colnames(credintervals[[1]])[1:4]
  for (n in 1:nreplic) {
  temp <- credintervals[[n]][2, ] - credintervals[[n]][1, ]
  precision[n, ] <- temp[1:4]
  }
  point <- matrix(NA, nreplic, nbsources)
  for (n in 1:nreplic) {
  point[n, ] <- apply(res[[n]][ ,1:4], 2, median)
  }
  return(list(credintervals, precision, point, resamples))
} 

# This function odifies the sample size only for one target source.
# same approach as above with the mean and sd from a normal distribution

onesource <- function(target, sours, nsamples, modifywhich, correct, nreplic, interv) {
  nbsources <- dim(sours)[1]

  res <- vector("list", nreplic)
  resamples <- vector("list", nreplic)
  for (n in 1:nreplic) {
    print(paste("calculates replicate data set", n), quote=FALSE)
    sours1 <- sours
    sours1[modifywhich, ] <- meansd.nd(sours[modifywhich, 2], sours[modifywhich, 3], 
     sours[modifywhich, 4], sours[modifywhich, 5], sours[modifywhich, 1], nsamples)
    resamples[[n]] <- sours1
    res[[n]] <- siarmcmcdirichletv4(target, sours1, correct)[[15]]
    rm(sours1)  
  }

  credintervals <- lapply(res, credint1, interva=interv) #list with a matrix with 2 lines for each source
  precision <- matrix(NA, nreplic, nbsources)
  colnames(precision) <- colnames(credintervals[[1]])[1:4]
  for (n in 1:nreplic) {
    temp <- credintervals[[n]][2, ] - credintervals[[n]][1, ]
    precision[n, ] <- temp[1:4]
    }
  point <- matrix(NA, nreplic, nbsources)
  for (n in 1:nreplic) {
    point[n, ] <- apply(res[[n]][ ,1:4], 2, median)
    }
  return(list(credintervals, precision, point, resamples))
} 

# This function runs allsources for several different sample sizes

Allsources <- function(targ, sour, nsampl, correc=NULL, nrepl=100, int=90) {
  nbsizes <- length(nsampl)
  creds <- vector("list", nbsizes)
  precisions <- vector("list", nbsizes)
  poins <- vector("list", nbsizes)
  Resamples <- vector("list", nbsizes) 
  for (m in 1:nbsizes) {
    temp <- allsources(target=targ, sours=sour, nsamples=nsampl[m], correct=correc, 
      nreplic=nrepl, interv=int)
    creds[[m]] <- temp[[1]] 
    precisions[[m]] <- temp[[2]] 
    poins[[m]] <- temp[[3]]
    Resamples[[m]] <- temp[[4]]
    } 
  return(list(creds, precisions, poins, Resamples, nsampl, int, "all sources"))
} 

# This function runs onesource for several different sample sizes

Onesource <- function(targ, sour, nsampl, modwhich, correc=NULL, nrepl=100, int=90) {
  nbsizes <- length(nsampl)
  creds <- vector("list", nbsizes)
  precisions <- vector("list", nbsizes)
  poins <- vector("list", nbsizes)
  Resamples <- vector("list", nbsizes) 
  for (m in 1:nbsizes) {
    temp <- onesource(target=targ, sours=sour, nsamples=nsampl[m], modifywhich=modwhich, 
      correct=correc, nreplic=nrepl, interv=int)
    creds[[m]] <- temp[[1]] 
    precisions[[m]] <- temp[[2]] 
    poins[[m]] <- temp[[3]]
    Resamples[[m]] <- temp[[4]]
    } 
  return(list(creds, precisions, poins, Resamples, nsampl, int, "one source", modwhich))  
} 

# This function starts from a mean and a standard deviation for the predator and
# calculates mixtures for different samples sizes of predators. 
# The mean and standard deviation for the predators should be in a vector with 
# Mean15N, Sd15N, Mean13C, Sd13C.

Consumer <- function(targ, sour, nsampl, correc=NULL, nrepl=100, int=90) {
  nbsizes <- length(nsampl)
  creds <- vector("list", nbsizes)
  precisions <- vector("list", nbsizes)
  poins <- vector("list", nbsizes)
  Resamples <- vector("list", nbsizes) 
  for (m in 1:nbsizes) {
    temp <- predators(target=targ, sours=sour, nsamples=nsampl[m], correct=correc, 
      nreplic=nrepl, interv=int)
    creds[[m]] <- temp[[1]] 
    precisions[[m]] <- temp[[2]] 
    poins[[m]] <- temp[[3]]
    Resamples[[m]] <- temp[[4]]
    } 
  return(list(creds, precisions, poins, Resamples, nsampl, int, "the consumer"))
} 

predators <- function(target, sours, nsamples, correct, nreplic, interv) {
  nbsources <- dim(sours)[1]

  res <- vector("list", nreplic)
  resamples <- vector("list", nreplic)
  for (n in 1:nreplic) {
    print(paste("calculates replicate data set", n), quote=FALSE)
    iso1 <- rnorm(nsamples, target[1], target[2])
    iso2 <- rnorm(nsamples, target[3], target[4])
    target1 <- cbind(iso1, iso2)
    resamples[[n]] <- target1
    #resamples[[n]] <- c(mean(tiso1), sd(tiso1), mean(tiso2), sd(tiso2))
    res[[n]] <- siarmcmcdirichletv4(target1, sours, correct)[[15]]
    }

  credintervals <- lapply(res, credint1, interva=interv) #list with a matrix with 2 lines for each source
  precision <- matrix(NA, nreplic, nbsources)
  colnames(precision) <- colnames(credintervals[[1]])[1:4]
  for (n in 1:nreplic) {
    temp <- credintervals[[n]][2, ] - credintervals[[n]][1, ]
    precision[n, ] <- temp[1:4]
    }
  point <- matrix(NA, nreplic, nbsources)
    for (n in 1:nreplic) {                                                                    
    point[n, ] <- apply(res[[n]][ ,1:4], 2, median)
    }
  return(list(credintervals, precision, point, resamples))
} 

#This function plots the results of the simulations

Plotsim <- function(res) {
  sizes <- res[[5]]
  nbsizes <- length(sizes)
  point <- res[[3]]
  nrepl <- dim(res[[2]][[1]])[1]
  nbsources <- dim(res[[2]][[1]])[2]
  precision <- array(NA, dim=c(nrepl, nbsources, nbsizes))
  estims <- array(NA, dim=c(nrepl, nbsources, nbsizes))
  for (n in 1:nbsizes) {
    precision[ , , n] <- res[[2]][[n]]
    estims[ , , n] <- res[[3]][[n]]
  }

  interval <- (nbsources-1)*0.2 + 1
  startp <- 0.2                             
  xpoints <- matrix(NA, nbsources, nbsizes) 
  for (i in 1:nbsources) {
    xpoints[i, ] <- (c(1:nbsizes) - 1)* interval + startp
    startp <- startp + 0.2
    }

  colour <- c("black", "red", "green", "grey", "brown", "gold", "pink", "olive", "turquoise", "violet")

  # plot credible intervals
  credtext <- paste("width of the ", res[[6]],"% credible intervals", sep="")
  plot(xpoints[1, ], apply(precision[ ,1, ], 2, mean), type="p", pch= 22, col="blue",
    main="influence of sample size on mixing model solutions", xlab="sample sizes", 
    ylab=credtext, ylim=c(0,1), xlim=c(0, (max(xpoints) + 0.2)), xaxt="n", 
    sub="for each source and sample size the min, mean and max are plotted")
  Axis(x = c(0, (max(xpoints) + 0.2)), at=xpoints[2, ], side=1, labels=sizes) 
  points(xpoints[1, ], apply(precision[ ,1, ], 2, min),pch= 25, col="blue")
  points(xpoints[1, ], apply(precision[ ,1, ], 2, max),pch= 24, col="blue")  
  for (j in 2:nbsources) {
    points(xpoints[j, ], apply(precision[ ,j, ], 2, mean),pch= 22, col=colour[j-1]) 
    points(xpoints[j, ], apply(precision[ ,j, ], 2, min),pch= 25, col=colour[j-1])
    points(xpoints[j, ], apply(precision[ ,j, ], 2, max),pch= 24, col=colour[j-1])      
  }
  if (length(res) == 8) {
    textwhich <- paste("for this plot source ", res[[8]], " has been modified")  
    mtext(textwhich, side = 4) 
  }
  toptext <- paste("sample sizes have been modified for ", res[[7]], sep="")
  mtext(toptext, side=3) 

  sourcenames <- colnames(res[[2]][[1]])
  legtext <- paste(sourcenames[1], ": blue", sep="")
  for (j in 2:nbsources){
    legtext <- paste(legtext, ", ", sourcenames[j], ": ", colour[j-1], sep="")
  }
  legend("topleft", legtext, bty="n")
  
  # plot medians
  plot(xpoints[1, ], apply(estims[ ,1, ], 2, mean), type="p", pch= 22, col="blue",
    main="influence of sample size on mixing model solutions", xlab="sample sizes", 
    ylab="median of posterior distributions", ylim=c(0,1), xlim=c(0, (max(xpoints) + 0.2)), xaxt="n", 
    sub="for each source and sample size the min, mean and max are plotted")
  Axis(x = c(0, (max(xpoints) + 0.2)), at=xpoints[2, ], side=1, labels=sizes) 
  points(xpoints[1, ], apply(estims[ ,1, ], 2, min),pch= 25, col="blue")
  points(xpoints[1, ], apply(estims[ ,1, ], 2, max),pch= 24, col="blue")  
  for (j in 2:nbsources) {
    points(xpoints[j, ], apply(estims[ ,j, ], 2, mean),pch= 22, col=colour[j-1]) 
    points(xpoints[j, ], apply(estims[ ,j, ], 2, min),pch= 25, col=colour[j-1])
    points(xpoints[j, ], apply(estims[ ,j, ], 2, max),pch= 24, col=colour[j-1])      
  }
  if (length(res) == 8) {
    textwhich <- paste("for this plot source ", res[[8]], " has been modified")  
    mtext(textwhich, side = 4) 
  }
  toptext <- paste("sample sizes have been modified for ", res[[7]], sep="")
  mtext(toptext, side=3) 

  legtext <- paste(sourcenames[1], ": blue", sep="")
  for (j in 2:nbsources){
    legtext <- paste(legtext, ", ", sourcenames[j], ": ", colour[j-1], sep="")
  }
  legend("topleft", legtext, bty="n")
}

# This function prints the results of the simulations to a text file called 
# samplesim-out.txt

Filesim <- function(res) {
  sizes <- res[[5]]
  nbsizes <- length(sizes)
  point <- res[[3]]
  nrepl <- dim(res[[2]][[1]])[1]           
  nbsources <- dim(res[[2]][[1]])[2]

  cat("Results of mixing model simulations modifying the sample sizes for ", res[[7]], "\n", file="samplesim-out.txt", sep="")
  cat("(mixing proportions were estimated with the R-package siar by Andrew Parnell)", "\n", 
    file="samplesim-out.txt", append=T, sep=" ")  
  if (length(res) == 8) {
  cat("samples sizes were modified for source", res[[8]], "\n", file="samplesim-out.txt", append=T, sep=" ")
    }
  cat("\n", file="samplesim-out.txt", append=T, sep=" ")
  cat("In this simulation", nrepl, "replicate data sets were analysed for samples sizes of", res[[5]],
    file="samplesim-out.txt", append=T, sep=" ")
  cat(".", "\n", "the width of the credible interval is ", res[[6]], ".", "\n", "\n", 
    file="samplesim-out.txt", append=T, sep="") 
  
  restab <- matrix(NA, (nrepl*nbsizes), (1 + 4*nbsources))
  restab[ , 1] <- sort(rep(sizes, nrepl)) 
  for(i in 0:(nbsources-1)){
    so <- 4
    si <- 0 
    for(j in 1:nbsizes){
      for(k in 1:nrepl){
        restab[si+k, (i*so)+2] <- res[[1]][[j]][[k]][1, i+1]
        restab[si+k, (i*so)+3] <- res[[1]][[j]][[k]][2, i+1]
      }
      restab[(si+1):(si+nrepl), (i*so)+4] <- res[[2]][[j]][ , i+1]
      restab[(si+1):(si+nrepl), (i*so)+5] <- res[[3]][[j]][ , i+1]
      si <- si + nrepl
    }
  }
  
  sourcenames <- colnames(res[[2]][[1]])
  cat("sample ", file="samplesim-out.txt", append=T, sep=" ") 
  for (i in 1:nbsources) {
    cat(rep(sourcenames[i], 4), " ", file="samplesim-out.txt", append=T, sep=" ")
    }
  cat("\n", file="samplesim-out.txt", append=T, sep=" ")
  cat("size ", file="samplesim-out.txt", append=T, sep=" ")
  for (i in 1:nbsources) {
    cat("lower", "upper", "width", "median ", file="samplesim-out.txt", append=T, sep=" ")
    }
  cat("\n", file="samplesim-out.txt", append=T, sep=" ")
  write.table(restab, file="samplesim-out.txt", append=T, sep=" ", quote=F, row.names=F, col.names=F) 
  cat("\n", "\n", file="samplesim-out.txt", append=T, sep=" ")

  cat("Resampled data", "\n", "\n", file="samplesim-out.txt", append=T, sep=" ")
  for (i in 1:nbsizes){
    cat("sample size", sizes[i], "\n", "\n", file="samplesim-out.txt", append=T, sep=" ")
    for (k in 1:nrepl) {
      cat("replicate nr", k, "\n", file="samplesim-out.txt", append=T, sep=" ")
      cat(colnames(res[[4]][[i]][[k]]), "\n", file="samplesim-out.txt", append=T, sep=" ")
      write.table(res[[4]][[i]][[k]], file="samplesim-out.txt", append=T, sep=" ", quote=F, col.names=F)
      cat("\n", file="samplesim-out.txt", append=T, sep=" ")
    }
  }
}                         